@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,7 @@ import org.apfloat.spi.Util;
 /**
  * Arbitrary precision calculator implementation.
  *
- * @version 1.10.0
+ * @version 1.11.0
  * @author Mikko Tommila
  */
 
@@ -174,6 +174,34 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.atanh((Apcomplex) x);
         }
 
+        @Override
+        public Number bernoulli(Number x)
+        {
+            if (!isLong(x))
+            {
+                throw new IllegalArgumentException("Bernoulli can only be used with a valid integer argument");
+            }
+            long n = x.longValue();
+            return AprationalMath.bernoulli(n);
+        }
+
+        @Override
+        public Number binomial(Number x, Number y)
+        {
+            return ApcomplexMath.binomial((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number catalan(Number x)
+        {
+            if (!isLong(x))
+            {
+                throw new IllegalArgumentException("Catalan can only be used with a valid integer argument");
+            }
+            long n = x.longValue();
+            return ApfloatMath.catalan(n);
+        }
+
         @Override
         public Number cbrt(Number x)
         {
@@ -198,6 +226,23 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.cosh((Apcomplex) x);
         }
 
+        @Override
+        public Number digamma(Number x)
+        {
+            return ApcomplexMath.digamma((Apcomplex) x);
+        }
+
+        @Override
+        public Number e(Number x)
+        {
+            if (!isLong(x))
+            {
+                throw new IllegalArgumentException("E can only be used with a valid integer argument");
+            }
+            long n = x.longValue();
+            return ApfloatMath.e(n);
+        }
+
         @Override
         public Number euler(Number x)
         {
@@ -238,6 +283,17 @@ public class ApfloatCalculatorImpl
             throw new IllegalArgumentException("Frac can only be used with scalar values");
         }
 
+        @Override
+        public Number glaisher(Number x)
+        {
+            if (!isLong(x))
+            {
+                throw new IllegalArgumentException("Glaisher can only be used with a valid integer argument");
+            }
+            long n = x.longValue();
+            return ApfloatMath.glaisher(n);
+        }
+
         @Override
         public Number gamma(Number x)
         {
@@ -256,6 +312,35 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.gamma((Apcomplex) x, (Apcomplex) y, (Apcomplex) z);
         }
 
+        @Override
+        public Number hypergeometric0F1(Number a, Number z)
+        {
+            return ApcomplexMath.hypergeometric0F1((Apcomplex) a, (Apcomplex) z);
+        }
+
+        @Override
+        public Number hypergeometric1F1(Number a, Number b, Number z)
+        {
+            return ApcomplexMath.hypergeometric1F1((Apcomplex) a, (Apcomplex) b, (Apcomplex) z);
+        }
+
+        @Override
+        public Number hypergeometric2F1(Number a, Number b, Number c, Number z)
+        {
+            return ApcomplexMath.hypergeometric2F1((Apcomplex) a, (Apcomplex) b, (Apcomplex) c, (Apcomplex) z);
+        }
+
+        @Override
+        public Number khinchin(Number x)
+        {
+            if (!isLong(x))
+            {
+                throw new IllegalArgumentException("Khinchin can only be used with a valid integer argument");
+            }
+            long n = x.longValue();
+            return ApfloatMath.khinchin(n);
+        }
+
         @Override
         public Number log(Number x)
         {
@@ -268,6 +353,12 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.log((Apcomplex) x, (Apcomplex) y);
         }
 
+        @Override
+        public Number logGamma(Number x)
+        {
+            return ApcomplexMath.logGamma((Apcomplex) x);
+        }
+
         @Override
         public Number max(Number x, Number y)
         {
@@ -334,7 +425,31 @@ public class ApfloatCalculatorImpl
         @Override
         public Number round(Number x, Number y)
         {
-            throw new IllegalArgumentException("Round can only be used with scalar values");
+            return roundToPrecision(x, y);
+        }
+
+        @Override
+        public Number roundToPrecision(Number x, Number y)
+        {
+            throw new IllegalArgumentException("Rounding can only be used with scalar values");
+        }
+
+        @Override
+        public Number roundToInteger(Number x)
+        {
+            throw new IllegalArgumentException("Rounding can only be used with scalar values");
+        }
+
+        @Override
+        public Number roundToPlaces(Number x, Number y)
+        {
+            throw new IllegalArgumentException("Rounding can only be used with scalar values");
+        }
+
+        @Override
+        public Number roundToMultiple(Number x, Number y)
+        {
+            throw new IllegalArgumentException("Rounding can only be used with scalar values");
         }
 
         @Override
@@ -391,6 +506,18 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.ulp((Apcomplex) x);
         }
 
+        @Override
+        public Number zeta(Number x)
+        {
+            return ApcomplexMath.zeta((Apcomplex) x);
+        }
+
+        @Override
+        public Number zeta(Number x, Number y)
+        {
+            return ApcomplexMath.zeta((Apcomplex) x, (Apcomplex) y);
+        }
+
         @Override
         public Number agm(Number x, Number y)
         {
@@ -645,18 +772,45 @@ public class ApfloatCalculatorImpl
         }
 
         @Override
-        public Number round(Number x, Number y)
+        public Number roundToPrecision(Number x, Number y)
         {
             if (!isLong(y))
             {
-                throw new IllegalArgumentException("Round can only be used with a valid integer argument");
+                throw new IllegalArgumentException("Rounding can only be used with a valid integer argument");
             }
-            return round(x, y.longValue());
+            return roundToPrecision(x, y.longValue());
         }
 
-        protected Number round(Number x, long precision)
+        protected Number roundToPrecision(Number x, long precision)
         {
-            return ApfloatMath.round((Apfloat) x, precision, RoundingMode.HALF_UP);
+            return ApfloatMath.roundToPrecision((Apfloat) x, precision, RoundingMode.HALF_UP);
+        }
+
+        @Override
+        public Number roundToInteger(Number x)
+        {
+            return ApfloatMath.roundToInteger((Apfloat) x, RoundingMode.HALF_UP);
+        }
+
+        @Override
+        public Number roundToPlaces(Number x, Number y)
+        {
+            if (!isLong(y))
+            {
+                throw new IllegalArgumentException("Rounding can only be used with a valid integer argument");
+            }
+            return roundToPlaces(x, y.longValue());
+        }
+
+        protected Number roundToPlaces(Number x, long precision)
+        {
+            return ApfloatMath.roundToPlaces((Apfloat) x, precision, RoundingMode.HALF_UP);
+        }
+
+        @Override
+        public Number roundToMultiple(Number x, Number y)
+        {
+            return ApfloatMath.roundToMultiple((Apfloat) x, (Apfloat) y, RoundingMode.HALF_UP);
         }
     }
 
@@ -699,6 +853,12 @@ public class ApfloatCalculatorImpl
             return AprationalMath.pow((Aprational) x, y);
         }
 
+        @Override
+        public Number binomial(Number x, Number y)
+        {
+            return AprationalMath.binomial((Aprational) x, (Aprational) y);
+        }
+
         @Override
         public Number hypot(Number x, Number y)
         {
@@ -722,9 +882,27 @@ public class ApfloatCalculatorImpl
         }
 
         @Override
-        protected Number round(Number x, long precision)
+        protected Number roundToPrecision(Number x, long precision)
         {
-            return AprationalMath.round((Aprational) x, precision, RoundingMode.HALF_EVEN);
+            return AprationalMath.roundToPrecision((Aprational) x, precision, RoundingMode.HALF_UP);
+        }
+
+        @Override
+        public Number roundToInteger(Number x)
+        {
+            return AprationalMath.roundToInteger((Aprational) x, RoundingMode.HALF_UP);
+        }
+
+        @Override
+        protected Number roundToPlaces(Number x, long places)
+        {
+            return AprationalMath.roundToPlaces((Aprational) x, places, RoundingMode.HALF_UP);
+        }
+
+        @Override
+        public Number roundToMultiple(Number x, Number y)
+        {
+            return AprationalMath.roundToMultiple((Aprational) x, (Aprational) y, RoundingMode.HALF_UP);
         }
 
         @Override
@@ -737,6 +915,16 @@ public class ApfloatCalculatorImpl
     private static class ApintFunctions
         extends AprationalFunctions
     {
+        @Override
+        public Number binomial(Number x, Number y)
+        {
+            if (isLong(x) && isLong(y))
+            {
+                return ApintMath.binomial(x.longValue(), y.longValue());
+            }
+            return ApintMath.binomial((Apint) x, (Apint) y);
+        }
+
         @Override
         public Number gcd(Number x, Number y)
         {
@@ -818,7 +1006,7 @@ public class ApfloatCalculatorImpl
         else if (x instanceof Apfloat)
         {
             Apfloat a = (Apfloat) x;
-            a = ApfloatMath.round(a.precision(Util.ifFinite(a.precision(), a.precision() + 1)), a.precision(), RoundingMode.HALF_UP);
+            a = ApfloatMath.roundToPrecision(a.precision(Util.ifFinite(a.precision(), a.precision() + 1)), a.precision(), RoundingMode.HALF_UP);
             text = a.toString(pretty);
         }
         else
