@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@ package org.apfloat;
 
 import java.util.Arrays;
 import java.util.Comparator;
+import java.util.Iterator;
 import java.util.Queue;
 import java.util.PriorityQueue;
 
@@ -35,7 +36,7 @@ import org.apfloat.spi.Util;
  *
  * @see ApfloatMath
  *
- * @version 1.10.1
+ * @version 1.11.0
  * @author Mikko Tommila
  */
 
@@ -108,7 +109,7 @@ public class ApcomplexMath
      *
      * @return <code>arctan(y / x)</code> from the appropriate branch, where <code>z = x + <i>i</i> y</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is zero.
+     * @exception ArithmeticException If <code>z</code> is zero.
      */
 
     public static Apfloat arg(Apcomplex z)
@@ -141,7 +142,7 @@ public class ApcomplexMath
      *
      * @return <code>z</code> to the <code>n</code>:th power, that is <code>z<sup>n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If both <code>z</code> and <code>n</code> are zero.
+     * @exception ArithmeticException If both <code>z</code> and <code>n</code> are zero.
      */
 
     public static Apcomplex pow(Apcomplex z, long n)
@@ -237,7 +238,7 @@ public class ApcomplexMath
      *
      * @return <code>n</code>:th root of <code>z</code>, that is <code>z<sup>1/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is zero.
+     * @exception ArithmeticException If <code>n</code> is zero.
      */
 
     public static Apcomplex root(Apcomplex z, long n)
@@ -256,7 +257,7 @@ public class ApcomplexMath
      *
      * @return <code>n</code>:th root of <code>z</code>, that is <code>z<sup>1/n</sup>e<sup>i2&pi;sk/n</sup></code> where <code>s</code> is the signum of the imaginary part of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is zero.
+     * @exception ArithmeticException If <code>n</code> is zero.
      *
      * @since 1.5
      */
@@ -323,7 +324,7 @@ public class ApcomplexMath
      *
      * @return Inverse <code>n</code>:th root of <code>z</code>, that is <code>z<sup>-1/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> or <code>n</code> is zero.
+     * @exception ArithmeticException If <code>z</code> or <code>n</code> is zero.
      */
 
     public static Apcomplex inverseRoot(Apcomplex z, long n)
@@ -342,7 +343,7 @@ public class ApcomplexMath
      *
      * @return Inverse <code>n</code>:th root of <code>z</code>, that is <code>z<sup>-1/n</sup>e<sup>-i2&pi;k/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> or <code>n</code> is zero.
+     * @exception ArithmeticException If <code>z</code> or <code>n</code> is zero.
      */
 
     public static Apcomplex inverseRoot(Apcomplex z, long n, long k)
@@ -591,7 +592,7 @@ public class ApcomplexMath
      *
      * @return All values of the <code>n</code>:th root of <code>z</code>, that is <code>z<sup>1/n</sup></code>, in the order of the angle.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is zero.
+     * @exception ArithmeticException If <code>n</code> is zero.
      *
      * @since 1.5
      */
@@ -698,7 +699,7 @@ public class ApcomplexMath
         // First check convergence
         while (precision < CONVERGING && precision < halfWorkingPrecision)
         {
-            Apcomplex t = a.add(b).divide(two);
+            Apcomplex t = limitPrecision(a.add(b)).divide(two);
             b = rightSqrt(a.multiply(b), t);
             a = t;
 
@@ -726,6 +727,11 @@ public class ApcomplexMath
         return ApfloatHelper.setPrecision(a.add(b).divide(two), targetPrecision);
     }
 
+    private static Apcomplex limitPrecision(Apcomplex z)
+    {
+        return z.precision(z.precision());
+    }
+
     private static Apcomplex rightSqrt(Apcomplex z, Apcomplex reference)
     {
         // See  D. A. Cox, "The Arithmetic-Geometric Mean of Gauss", L'Enseignement Mathematique, Vol. 30, 1984, pp. 275-330
@@ -761,15 +767,19 @@ public class ApcomplexMath
      *
      * @return Natural logarithm of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is zero.
+     * @exception ArithmeticException If <code>z</code> is zero.
      */
 
     public static Apcomplex log(Apcomplex z)
         throws ArithmeticException, ApfloatRuntimeException
     {
-        if (z.real().signum() >= 0 && z.imag().signum() == 0)
+        if (z.imag().signum() == 0)
         {
-            return ApfloatMath.log(z.real());
+            if (z.real().signum() >= 0)
+            {
+                return ApfloatMath.log(z.real());
+            }
+            return new Apcomplex(ApfloatMath.log(z.real().negate()), ApfloatMath.pi(z.precision(), z.radix()));
         }
 
         // Calculate the log using 1 / radix <= |z| < 1 and the log addition formula
@@ -851,7 +861,7 @@ public class ApcomplexMath
      *
      * @return Base-<code>w</code> logarithm of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> or <code>w</code> is zero.
+     * @exception ArithmeticException If <code>z</code> or <code>w</code> is zero.
      *
      * @since 1.6
      */
@@ -1147,7 +1157,7 @@ public class ApcomplexMath
      *
      * @return <code>z<sup>w</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If both <code>z</code> and <code>w</code> are zero.
+     * @exception ArithmeticException If both <code>z</code> and <code>w</code> are zero.
      */
 
     public static Apcomplex pow(Apcomplex z, Apcomplex w)
@@ -1296,7 +1306,7 @@ public class ApcomplexMath
      *
      * @return Inverse tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z == <i>i</i></code>.
+     * @exception ArithmeticException If <code>z == <i>i</i></code>.
      */
 
     public static Apcomplex atan(Apcomplex z)
@@ -1329,7 +1339,7 @@ public class ApcomplexMath
      *
      * @return Inverse hyperbolic tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is 1 or -1.
+     * @exception ArithmeticException If <code>z</code> is 1 or -1.
      */
 
     public static Apcomplex atanh(Apcomplex z)
@@ -1450,18 +1460,29 @@ public class ApcomplexMath
      *
      * @return Tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is &pi;/2 + n &pi; where n is an integer.
+     * @exception ArithmeticException If <code>z</code> is &pi;/2 + n &pi; where n is an integer.
      */
 
     public static Apcomplex tan(Apcomplex z)
         throws ArithmeticException, ApfloatRuntimeException
+    {
+        return tan(z, z.imag().signum() < 0);
+    }
+
+    static Apcomplex tanFixedPrecision(Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return tan(z, z.imag().signum() > 0);
+    }
+
+    static Apcomplex tan(Apcomplex z, boolean negate)
+        throws ArithmeticException, ApfloatRuntimeException
     {
         if (z.imag().signum() == 0)
         {
             return ApfloatMath.tan(z.real());
         }
 
-        boolean negate = z.imag().signum() > 0;
         z = (negate ? z.negate() : z);
 
         Apfloat one = new Apfloat(1, Apfloat.INFINITE, z.radix()),
@@ -1481,7 +1502,7 @@ public class ApcomplexMath
      *
      * @return Hyperbolic tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is <i>i</i> (&pi;/2 + n &pi;) where n is an integer.
+     * @exception ArithmeticException If <code>z</code> is <i>i</i> (&pi;/2 + n &pi;) where n is an integer.
      */
 
     public static Apcomplex tanh(Apcomplex z)
@@ -1515,6 +1536,22 @@ public class ApcomplexMath
         return (negate ? w.negate() : w);
     }
 
+    static Apcomplex cot(Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        boolean negate = z.imag().signum() < 0;
+        z = (negate ? z.negate() : z);
+
+        Apfloat one = new Apfloat(1, Apfloat.INFINITE, z.radix()),
+                two = new Apfloat(2, Apfloat.INFINITE, z.radix());
+        Apcomplex i = new Apcomplex(Apfloat.ZERO, one),
+                  w = expNoLoP(two.multiply(i).multiply(z));
+
+        w = i.multiply(two.multiply(w).divide(w.subtract(one)).subtract(one));
+
+        return (negate ? w.negate() : w);
+    }
+
     /**
      * Lambert W function. The W function gives the solution to the equation
      * <code>W e<sup>W</sup> = z</code>. Also known as the product logarithm.<p>
@@ -1542,7 +1579,7 @@ public class ApcomplexMath
      *
      * @return <code>W<sub>k</sub>(z)</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is zero and <code>k</code> is not zero.
+     * @exception ArithmeticException If <code>z</code> is zero and <code>k</code> is not zero.
      *
      * @see #w(Apcomplex)
      *
@@ -1655,8 +1692,9 @@ public class ApcomplexMath
     /**
      * Gamma function.<p>
      *
+     * @implNote
      * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
-     * The asymptotic complexity is something like O(n<sup>2</sup>log&nbsp;n) and it is
+     * The asymptotic complexity is at least O(n<sup>2</sup>log&nbsp;n) and it is
      * impractically slow beyond a precision of a few thousand digits. At the time of
      * implementation no generic fast algorithm is known for the gamma function.
      *
@@ -1712,6 +1750,7 @@ public class ApcomplexMath
         }
         if (z.real().signum() < 0)
         {
+            // Use reflection formula, see e.g. https://functions.wolfram.com/GammaBetaErf/Gamma/16/03/01/
             z = z.negate();
             Apfloat pi = ApfloatMath.pi(precision, radix);
             return pi.negate().divide(z.multiply(sin(pi.multiply(z))).multiply(gamma(z)));
@@ -1762,8 +1801,9 @@ public class ApcomplexMath
     /**
      * Incomplete gamma function.<p>
      *
+     * @implNote
      * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
-     * The asymptotic complexity is something like O(n<sup>2</sup>log&nbsp;n) and it is
+     * The asymptotic complexity is at least O(n<sup>2</sup>log&nbsp;n) and it is
      * impractically slow beyond a precision of a few thousand digits. At the time of
      * implementation no generic fast algorithm is known for the incomplete gamma function.
      *
@@ -1790,8 +1830,9 @@ public class ApcomplexMath
      *
      * The lower gamma function can be calculated with: <code>&gamma;(a, z) = &Gamma;(a, 0, z)</code><p>
      *
+     * @implNote
      * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
-     * The asymptotic complexity is something like O(n<sup>2</sup>log&nbsp;n) and it is
+     * The asymptotic complexity is at least O(n<sup>2</sup>log&nbsp;n) and it is
      * impractically slow beyond a precision of a few thousand digits. At the time of
      * implementation no generic fast algorithm is known for the incomplete gamma function.
      *
@@ -1812,6 +1853,543 @@ public class ApcomplexMath
         return IncompleteGammaHelper.gamma(a, z0, z1);
     }
 
+    /**
+     * Logarithm of the gamma function. Note that this function has a different branch
+     * structure than <code>log(gamma(z))</code>.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * The asymptotic complexity is at least O(n<sup>2</sup>log&nbsp;n) and it is
+     * impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the gamma function.
+     *
+     * @param z The argument.
+     *
+     * @return <code>log&Gamma;(z)</code>
+     *
+     * @throws ArithmeticException If <code>z</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex logGamma(Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        // |B_2n| ~ 4 sqrt(pi n) (n / (pi e))^2n
+        // Stirling's series (does not converge)
+        // B_2n / (2n(2n-1)z^(2n-1))
+
+        // The sum diverges, but depending on how large re(z) is, the terms initially get smaller and smaller,
+        // until they start getting bigger and bigger (and grow to infinity)
+
+        // By truncating the sum at the point where the terms are the smallest, we can get a good approximation
+
+        // Thus we can calculate which is the smallest term, given any z
+        // The larger re(z) is, the larger the n of the term is, and the smaller the term is
+        // For higher precision we need more terms, and a larger re(z)
+        // Use the recurrence formula to move re(z) to be as large as needed
+        // For negative re(z) use first the reflection formula
+
+        // To calculate how many terms of the sum we need, and how big should re(z) be:
+        // Use the asymptotic formula for B_2n (which is good enough for n >= 3)
+        // The term in the sum is B_2n / (2n(2n-1)z^(2n-1))
+        //
+        // which is approximately
+        //
+        //  z (E Pi z / n)^(-2n) / (2n (2n - 1))
+        //
+        // Take derivative with respect to n and solve when it's zero
+        //
+        // z = (n exp((4n - 1) / (2n - 4n^2))) / Pi
+        //
+        // Substitute back to what the term is at that point
+        //
+        // exp(-(2n - 1)^2 / (2n)) / (2 Pi (2 n - 1))
+        //
+        // For precision p in base b, we want that term to be equal to b^-p, solve that for n
+        // (cannot be solved but approximately 2n ~ 2n-1 for large n, then it can be solved)
+        //
+        // n = 1/2 (1 + W(b^p / (2 Pi)))
+        //
+        // Use formula further above to get corresponding value for z
+        //
+        // W is Lambert's W function
+        // W can be approximated by log(z) - log(log(z))
+        // Followed possibly by iteration(s) of w = w/(1 + w) (1 + log(x/w))
+        //
+        // The bernoulli number factor of ~sqrt(n) has been ignored in the above calculations,
+        // compensate by adding a few digits of extra precision
+
+        long precision = z.precision();
+        if (z.imag().signum() == 0)
+        {
+            if (z.real().signum() == 0)
+            {
+                throw new ArithmeticException("Log gamma of zero");
+            }
+            if (z.real().isInteger() && z.real().signum() < 0)
+            {
+                throw new ArithmeticException("Log gamma of negative integer");
+            }
+        }
+        if (precision == Apfloat.INFINITE)
+        {
+            throw new InfiniteExpansionException("Cannot calculate log gamma function to infinite precision");
+        }
+
+        int radix = z.radix();
+        Apint one = Apint.ONES[radix],
+              two = new Apint(2, radix);
+        long workingPrecision = ApfloatHelper.extendPrecision(precision);
+        Apfloat pi = ApfloatMath.pi(workingPrecision, radix);
+
+        if (z.real().signum() <= 0)
+        {
+            // Use reflection formula
+            if (z.scale() < -precision)
+            {
+                // See e.g. https://functions.wolfram.com/GammaBetaErf/LogGamma/16/01/01/
+                // Note that with z so small, now sin(z) ~= z, accurate to the precision of z
+                return log(pi).subtract(log(pi.multiply(z))).subtract(logGamma(z.negate())).subtract(log(z.negate()));
+            }
+            // See: Arbitrary-precision computation of the gamma function, Fredrik Johansson, https://arxiv.org/pdf/2109.08392.pdf
+            return log(pi).subtract(logSin(z)).subtract(logGamma(one.subtract(z)));
+        }
+
+        double adjust = Math.log(precision) + 1,    // Adjustment for the sqrt(n) factor in bernoulli numbers
+               w = (precision + adjust) * Math.log(radix) - Math.log(2 * Math.PI);
+        long n = (long) Math.ceil(0.5 * (1 + w - Math.log(w)));
+        Apfloat zReal = new Apfloat(n * Math.exp((4. * n - 1) / (2 * n - 4. * n * n)) / Math.PI, precision, radix);
+        Apcomplex s = Apcomplex.ZERO;
+        if (z.real().compareTo(zReal) < 0)
+        {
+            long N = zReal.subtract(z.real()).roundAway().longValueExact();
+            // Use recurrence formula
+            s = s.subtract(logPochhammer(z, N));
+            z = z.add(new Apfloat(N, precision, radix));
+        }
+
+        s = s.add(z.subtract(new Aprational(one, two)).multiply(log(z))).subtract(z).add(log(two.multiply(pi)).divide(two));
+        Apcomplex z2 = z.multiply(z),
+                  zp = z;
+        Iterator<Aprational> bernoulli2 = AprationalMath.bernoullis2(n, radix);
+        for (long k = 1; k <= n; k++)
+        {
+            long k2 = Math.multiplyExact(k,  2);
+            Apcomplex term = bernoulli2.next().precision(workingPrecision).divide(new Apint(k2, radix).multiply(new Apint(k2 - 1, radix)).multiply(zp));
+            if (k < n)
+            {
+                zp = zp.multiply(z2);
+            }
+
+            long[] matchingPrecisionsReal = ApfloatHelper.getMatchingPrecisions(s.real(), term.real());
+            long[] matchingPrecisionsImag = ApfloatHelper.getMatchingPrecisions(s.imag(), term.imag());
+            if (matchingPrecisionsReal[1] == 0 && matchingPrecisionsImag[1] == 0)
+            {
+                // The rest of the terms would be insignificantly small
+                break;
+            }
+
+            s = s.add(term);
+        }
+
+        return s;
+    }
+
+    // log(sin(pi z)) with correct branch structure
+    private static Apcomplex logSin(Apcomplex z)
+    {
+        // See https://arxiv.org/pdf/2109.08392.pdf Arbitrary-precision computation of the gamma function by Fredrik Johansson
+        long precision = z.precision(),
+             workingPrecision = ApfloatHelper.extendPrecision(precision);
+        int radix = z.radix();
+        Apint n = z.real().floor(),
+              one = Apint.ONES[radix],
+              two = new Apint(2, radix);
+        Apfloat half = new Aprational(one, two).precision(workingPrecision),
+                pi = ApfloatMath.pi(workingPrecision, radix);
+        Apcomplex i = new Apcomplex(Apint.ZERO, Apint.ONES[radix]);
+        Apcomplex offset = n.multiply(pi).multiply(i);
+        assert (z.real().signum() <= 0);
+        if (z.imag().signum() >= 0)
+        {
+            offset = offset.negate();
+        }
+        z = z.subtract(n);
+        Apcomplex ls;
+        if (z.imag().compareTo(one) > 0)
+        {
+            ls = log(half.multiply(one.subtract(expNoLoP(two.multiply(i).multiply(pi).multiply(z))))).subtract(i.multiply(pi).multiply(z.subtract(half)));
+        }
+        else if (z.imag().compareTo(one.negate()) < 0)
+        {
+            ls = log(half.multiply(one.subtract(expNoLoP(two.negate().multiply(i).multiply(pi).multiply(z))))).add(i.multiply(pi).multiply(z.subtract(half)));
+        }
+        else
+        {
+            ls = log(sin(pi.multiply(z)));
+        }
+        return ls.add(offset);
+    }
+
+    private static Apcomplex expNoLoP(Apcomplex z)
+    {
+        // Avoid loss of precision if z is too big
+        if (z.real().signum() < 0)
+        {
+            if (z.real().scale() > 1)
+            {
+                if (z.real().precision() <= z.real().scale() - 1)
+                {
+                    z = new Apcomplex(z.real().precision(z.real().scale()), z.imag());
+                }
+            }
+        }
+        return exp(z);
+    }
+
+    private static Apcomplex logPochhammer(Apcomplex z, long n)
+    {
+        // Hare's algorithm
+        boolean conj = (z.imag().signum() < 0);
+        if (conj)
+        {
+            z = z.conj();
+        }
+        int radix = z.radix();
+        Apcomplex s = z;
+        long m = 0;
+        for (long k = 1; k < n; k++)
+        {
+            Apcomplex t = s.multiply(z.add(new Apint(k, radix)));
+            if (s.imag().signum() >= 0 && t.imag().signum() < 0)
+            {
+                 m += 2;
+            }
+            s = t;
+        }
+        if (s.real().signum() < 0)
+        {
+            if (s.imag().signum() >= 0)
+            {
+                m++;
+            }
+            else
+            {
+                m--;
+            }
+            s = s.negate();
+        }
+        Apcomplex i = new Apcomplex(Apint.ZERO, Apint.ONES[radix]);
+        Apfloat pi = ApfloatMath.pi(z.precision(), radix);
+        Apcomplex result = log(s).add(pi.multiply(i).multiply(new Apint(m, radix)));
+        return (conj ? result.conj() : result);
+    }
+
+    /**
+     * Digamma function.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * The asymptotic complexity is at least O(n<sup>2</sup>log&nbsp;n) and it is
+     * impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the digamma function.
+     *
+     * @param z The argument.
+     *
+     * @return <code>&psi;(z)</code>
+     *
+     * @throws ArithmeticException If <code>z</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex digamma(Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        // digamma(x) = digamma(1-x) - pi cot(pi x)
+        // |B_2n| ~ 4 sqrt(pi n) (n / (pi e))^2n
+        // digamma(z) ~ ln(z) - 1 / 2z - sum (j=1, infinity, B_2j / (2j z^2j)), re(z) > 0
+        // digamma(z) = digamma(z + N) - sum (k=0, N-1, 1 / (z + k))
+
+        // The sum diverges, but depending on how large re(z) is, the terms initially get smaller and smaller,
+        // until they start getting bigger and bigger (and grow to infinity)
+
+        // By truncating the sum at the point where the terms are the smallest, we can get a good approximation
+
+        // Thus we can calculate which is the smallest term, given any z
+        // The larger re(z) is, the larger the n of the term is, and the smaller the term is
+        // For higher precision we need more terms, and a larger re(z)
+        // Use the recurrence formula to move re(z) to be as large as needed
+        // For negative re(z) use first the reflection formula
+
+        // To calculate how many terms of the sum we need, and how big should re(z) be:
+        // Use the asymptotic formula for B_2n (which is good enough for n >= 3)
+        // The term in the sum is B_2n / (2n z^2n)
+        //
+        // which is approximately
+        //
+        // (n / (pi e))^2n / (2n z^2n)
+        // =
+        // 1/2 n^(2n-1) (e pi z)^(-2n)
+        // =
+        // 1/(2n) (e pi z / n)^(-2n)
+        //
+        // Take derivative with respect to n and solve when it's zero
+        //
+        // n = 1/(2 W(1/(2 pi z)))
+        // so then (solve for z)
+        // z = e^(-1/(2n)) n / pi
+        //
+        // Substitute back to what the term is at that point
+        //
+        // e^(1-2n) / (2n)
+        //
+        // For precision p in base b, we want that term to be equal to b^-p, solve that for n
+        //
+        // n = 1/2 W(b^p e)
+        //
+        // Use formula further above to get corresponding value for z
+        //
+        // W is Lambert's W function
+        // W can be approximated by log(z) - log(log(z))
+        // Followed possibly by iteration(s) of w = w/(1 + w) (1 + log(x/w))
+        //
+        // The bernoulli number factor of ~sqrt(n) has been ignored in the above calculations,
+        // compensate by adding a few digits of extra precision
+
+        long precision = z.precision();
+        int radix = z.radix();
+        Apint one = Apint.ONES[radix];
+
+        if (z.real().signum() <= 0)
+        {
+            if (z.real().isInteger() && z.imag().signum() == 0)
+            {
+                throw new ArithmeticException("Digamma of nonpositive integer");
+            }
+            if (precision == Apfloat.INFINITE)
+            {
+                throw new InfiniteExpansionException("Cannot calculate digamma function to infinite precision");
+            }
+            Apfloat pi = ApfloatMath.pi(precision, radix);
+            // Use reflection formula, see e.g. https://functions.wolfram.com/GammaBetaErf/PolyGamma/16/01/01/
+            if (z.scale() < -precision)
+            {
+                return digamma(z.negate()).subtract(pi.multiply(cot(pi.multiply(z)))).subtract(one.divide(z));
+            }
+            return digamma(one.subtract(z)).subtract(pi.multiply(cot(pi.multiply(z))));
+        }
+        if (precision == Apfloat.INFINITE)
+        {
+            throw new InfiniteExpansionException("Cannot calculate digamma function to infinite precision");
+        }
+
+        double adjust = Math.log(precision) + 1,    // Adjustment for the sqrt(n) factor in bernoulli numbers
+               w = (precision + adjust) * Math.log(radix) + 1;
+        long n = (long) Math.ceil(0.5 * (w - Math.log(w)));
+        Apfloat zReal = new Apfloat(Math.exp(-0.5 / n) * n / Math.PI, precision, radix);
+        Apcomplex s = Apfloat.ZERO;
+        if (z.real().compareTo(zReal) < 0)
+        {
+            long N = zReal.subtract(z.real()).roundAway().longValueExact();
+            // Use recurrence formula
+            for (long k = 0; k < N; k++)
+            {
+                s = s.subtract(one.divide(z.add(new Apint(k, radix))));
+            }
+            z = z.add(new Apfloat(N, precision, radix));
+        }
+
+        Apint two = new Apint(2, radix);
+        s = s.add(log(z)).subtract(one.divide(two.multiply(z)));
+        Apcomplex z2 = z.multiply(z),
+                  zp = one;
+        Iterator<Aprational> bernoulli2 = AprationalMath.bernoullis2(n, radix);
+        for (long k = 1; k <= n; k++)
+        {
+            long k2 = Math.multiplyExact(k,  2);
+            zp = zp.multiply(z2);
+            Apcomplex term = bernoulli2.next().precision(precision).divide(new Apint(k2, radix).multiply(zp));
+
+            long[] matchingPrecisionsReal = ApfloatHelper.getMatchingPrecisions(s.real(), term.real());
+            long[] matchingPrecisionsImag = ApfloatHelper.getMatchingPrecisions(s.imag(), term.imag());
+            if (matchingPrecisionsReal[1] == 0 && matchingPrecisionsImag[1] == 0)
+            {
+                // The rest of the terms would be insignificantly small
+                break;
+            }
+
+            s = s.subtract(term);
+        }
+
+        return s;
+    }
+
+    /**
+     * Binomial coefficient. Calculated using the {@link #gamma(Apcomplex)} function.
+     *
+     * @param n The first argument.
+     * @param k The second argument.
+     *
+     * @return <math xmlns="http://www.w3.org/1998/Math/MathML">
+     *           <mrow>
+     *             <mo>(</mo>
+     *               <mfrac linethickness="0">
+     *                 <mi>n</mi>
+     *                 <mi>k</mi>
+     *               </mfrac>
+     *             <mo>)</mo>
+     *           </mrow>
+     *         </math>
+     *
+     * @throws ArithmeticException If <code>n</code> is a negative integer and <code>k</code> is noninteger.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex binomial(Apcomplex n, Apcomplex k)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        if (n.imag().signum() == 0 && k.imag().signum() == 0)
+        {
+            return ApfloatMath.binomial(n.real(), k.real());
+        }
+        Apcomplex nk = n.subtract(k);
+        if (k.isInteger() && k.real().signum() < 0 ||
+            nk.isInteger() && nk.real().signum() < 0)
+        {
+            // The divisor is infinity (but the dividend isn't) so we get zero
+            return Apcomplex.ZEROS[n.radix()];
+        }
+        Apint one = Apint.ONES[n.radix()];
+        return gamma(n.add(one)).divide(gamma(k.add(one)).multiply(gamma(nk.add(one))));
+    }
+
+    /**
+     * Riemann zeta function.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * It is impractically slow beyond a precision of a few hundred digits. At the time of
+     * implementation no generic fast algorithm is known for the zeta function.
+     *
+     * @param s The argument.
+     *
+     * @return <code>&zeta;(s)</code>
+     *
+     * @throws ArithmeticException If <code>s</code> is <code>1</code>.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex zeta(Apcomplex s)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return ZetaHelper.zeta(s);
+    }
+
+    /**
+     * Hurwitz zeta function.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * It is impractically slow beyond a precision of a few hundred digits. At the time of
+     * implementation no generic fast algorithm is known for the zeta function.
+     *
+     * @param s The first argument.
+     * @param a The second argument.
+     *
+     * @return <code>&zeta;(s, a)</code>
+     *
+     * @throws ArithmeticException If <code>s</code> is <code>1</code> or if <code>a</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex zeta(Apcomplex s, Apcomplex a)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return HurwitzZetaHelper.zeta(s, a);
+    }
+
+    /**
+     * Confluent hypergeometric function <i><sub>0</sub>F<sub>1</sub></i>.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * It is impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the function.
+     *
+     * @param a The first argument.
+     * @param z The second argument.
+     *
+     * @return <i><sub>0</sub>F<sub>1</sub>(; a; z)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex hypergeometric0F1(Apcomplex a, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return HypergeometricHelper.hypergeometricPFQ(new Apcomplex[0], new Apcomplex[] { a }, z);
+    }
+
+    /**
+     * Kummer confluent hypergeometric function <i><sub>1</sub>F<sub>1</sub></i>.
+     * Also known as the confluent hypergeometric function of the first kind.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * It is impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the function.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param z The third argument.
+     *
+     * @return <i><sub>1</sub>F<sub>1</sub>(a; b; z)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex hypergeometric1F1(Apcomplex a, Apcomplex b, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return HypergeometricHelper.hypergeometricPFQ(new Apcomplex[] { a }, new Apcomplex[] { b }, z);
+    }
+
+    /**
+     * Hypergeometric function <i><sub>2</sub>F<sub>1</sub></i>.
+     * Also known as the Gaussian or ordinary hypergeometric function.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * It is impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the function.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param c The third argument.
+     * @param z The fourth argument.
+     *
+     * @return <i><sub>2</sub>F<sub>1</sub>(a, b; c; z)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apcomplex hypergeometric2F1(Apcomplex a, Apcomplex b, Apcomplex c, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return HypergeometricHelper.hypergeometricPFQ(new Apcomplex[] { a, b }, new Apcomplex[] { c }, z);
+    }
+
     /**
      * Returns the unit in the last place of the argument, considering the
      * scale and precision. This is maximum of the ulps of the real and
