@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,7 +31,7 @@ import java.util.Random;
 /**
  * Various mathematical functions for arbitrary precision integers.
  *
- * @version 1.10.1
+ * @version 1.11.0
  * @author Mikko Tommila
  */
 
@@ -49,7 +49,7 @@ public class ApintMath
      *
      * @return <code>x</code> to the <code>n</code>:th power, that is <code>x<sup>n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If both <code>x</code> and <code>n</code> are zero.
+     * @exception ArithmeticException If both <code>x</code> and <code>n</code> are zero.
      */
 
     public static Apint pow(Apint x, long n)
@@ -104,7 +104,7 @@ public class ApintMath
      *
      * @return An array of two apints: <code>[q, r]</code>, where <code>q<sup>2</sup> + r = x</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> is negative.
+     * @exception ArithmeticException If <code>x</code> is negative.
      */
 
     public static Apint[] sqrt(Apint x)
@@ -138,7 +138,7 @@ public class ApintMath
      *
      * @return An array of two apints: <code>[q, r]</code>, where <code>q<sup>n</sup> + r = x</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> and <code>x</code> are zero, or <code>x</code> is negative and <code>n</code> is even.
+     * @exception ArithmeticException If <code>n</code> and <code>x</code> are zero, or <code>x</code> is negative and <code>n</code> is even.
      */
 
     public static Apint[] root(Apint x, long n)
@@ -341,7 +341,7 @@ public class ApintMath
      *
      * @return An array of two apints: <code>[quotient, remainder]</code>, that is <code>[x / y, x % y]</code>.
      *
-     * @exception java.lang.ArithmeticException In case the divisor is zero.
+     * @exception ArithmeticException In case the divisor is zero.
      */
 
     public static Apint[] div(Apint x, Apint y)
@@ -507,7 +507,7 @@ public class ApintMath
      *
      * @return <code>a<sup>b</sup> mod m</code>
      *
-     * @exception java.lang.ArithmeticException If the exponent is negative but the GCD of <code>a</code> and <code>m</code> is not 1 and the modular inverse does not exist.
+     * @exception ArithmeticException If the exponent is negative but the GCD of <code>a</code> and <code>m</code> is not 1 and the modular inverse does not exist.
      */
 
     public static Apint modPow(Apint a, Apint b, Apint m)
@@ -615,8 +615,8 @@ public class ApintMath
      *
      * @return <code>n!</code>
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is negative.
-     * @exception java.lang.NumberFormatException If the default radix is not valid.
+     * @exception ArithmeticException If <code>n</code> is negative.
+     * @exception NumberFormatException If the default radix is not valid.
      *
      * @since 1.1
      */
@@ -635,8 +635,8 @@ public class ApintMath
      *
      * @return <code>n!</code>
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is negative.
-     * @exception java.lang.NumberFormatException If the radix is not valid.
+     * @exception ArithmeticException If <code>n</code> is negative.
+     * @exception NumberFormatException If the radix is not valid.
      *
      * @since 1.1
      */
@@ -647,6 +647,199 @@ public class ApintMath
         return new Apint(ApfloatMath.factorial(n, Apfloat.INFINITE, radix));
     }
 
+    /**
+     * Binomial coefficient. Uses the default radix.
+     *
+     * @param n The first argument.
+     * @param k The second argument.
+     *
+     * @return <math xmlns="http://www.w3.org/1998/Math/MathML">
+     *           <mrow>
+     *             <mo>(</mo>
+     *               <mfrac linethickness="0">
+     *                 <mi>n</mi>
+     *                 <mi>k</mi>
+     *               </mfrac>
+     *             <mo>)</mo>
+     *           </mrow>
+     *         </math>
+     *
+     * @since 1.11.0
+     */
+
+    public static Apint binomial(long n, long k)
+        throws ApfloatRuntimeException
+    {
+        ApfloatContext ctx = ApfloatContext.getContext();
+        int radix = ctx.getDefaultRadix();
+
+        return binomial(n, k, radix);
+    }
+
+    /**
+     * Binomial coefficient. Uses the specified radix.
+     *
+     * @param n The first argument.
+     * @param k The second argument.
+     * @param radix The radix.
+     *
+     * @return <math xmlns="http://www.w3.org/1998/Math/MathML">
+     *           <mrow>
+     *             <mo>(</mo>
+     *               <mfrac linethickness="0">
+     *                 <mi>n</mi>
+     *                 <mi>k</mi>
+     *               </mfrac>
+     *             <mo>)</mo>
+     *           </mrow>
+     *         </math>
+     *
+     * @throws NumberFormatException If the radix is not valid.
+     *
+     * @since 1.11.0
+     */
+
+    public static Apint binomial(long n, long k, int radix)
+        throws NumberFormatException, ApfloatRuntimeException
+    {
+        // See https://mathworld.wolfram.com/BinomialCoefficient.html for the logic on negative values
+        boolean negate = false;
+        if (n < 0)
+        {
+            if (k >= 0)
+            {
+                n = Math.subtractExact(k, n) - 1;
+            }
+            else if (k <= n)
+            {
+                long n1 = -k - 1;
+                k = n - k;
+                n = n1;
+            }
+            else
+            {
+                return Apint.ZEROS[radix];
+            }
+            negate = ((k & 1) == 1);
+        }
+        else if (k < 0)
+        {
+            k = Math.subtractExact(n, k);
+        }
+        if (k < 0 || k > n)
+        {
+            return Apint.ZEROS[radix];
+        }
+        assert (n >= 0);
+        assert (k >= 0);
+        if (k > n / 2)
+        {
+            // Optimize performance
+            k = n - k;
+        }
+        Apint b = pochhammer(n - k + 1, k, radix).divide(factorial(k, radix));
+        return (negate ? b.negate() : b);
+    }
+
+    /**
+     * Binomial coefficient.
+     *
+     * @param n The first argument.
+     * @param k The second argument.
+     *
+     * @return <math xmlns="http://www.w3.org/1998/Math/MathML">
+     *           <mrow>
+     *             <mo>(</mo>
+     *               <mfrac linethickness="0">
+     *                 <mi>n</mi>
+     *                 <mi>k</mi>
+     *               </mfrac>
+     *             <mo>)</mo>
+     *           </mrow>
+     *         </math>
+     *
+     * @since 1.11.0
+     */
+
+    public static Apint binomial(Apint n, Apint k)
+        throws ApfloatRuntimeException
+    {
+        // See https://mathworld.wolfram.com/BinomialCoefficient.html for the logic on negative values
+        boolean negate = false;
+        int radix = n.radix();
+        Apint one = Apint.ONES[radix],
+              two = new Apint(2, radix);
+        if (n.signum() < 0)
+        {
+            if (k.signum() >= 0)
+            {
+                n = k.subtract(n).subtract(one);
+            }
+            else if (k.compareTo(n) <= 0)
+            {
+                Apint n1 = k.negate().subtract(one);
+                k = n.subtract(k);
+                n = n1;
+            }
+            else
+            {
+                return Apint.ZEROS[radix];
+            }
+            negate = (k.mod(two).signum() != 0);
+        }
+        else if (k.signum() < 0)
+        {
+            k = n.subtract(k);
+        }
+        if (k.signum() < 0 || k.compareTo(n) > 0)
+        {
+            return Apint.ZEROS[radix];
+        }
+        assert (n.signum() >= 0);
+        assert (k.signum() >= 0);
+        if (k.compareTo(n.divide(two)) > 0)
+        {
+            // Optimize performance
+            k = n.subtract(k);
+        }
+        Apint b = pochhammer(n.subtract(k).add(one), k).divide(factorial(k.longValueExact(), radix));
+        return (negate ? b.negate() : b);
+    }
+
+    // Product of the numbers n * (n + 1) * (n + 2) * ... * (n + m - 1)
+    private static Apint pochhammer(long n, long m, int radix)
+    {
+        assert (m >= 0);
+        if (m == 0)
+        {
+            return Apint.ONES[radix];
+        }
+        if (m == 1)
+        {
+            return new Apint(n, radix);
+        }
+        long k = m >>> 1;
+        return pochhammer(n, k, radix).multiply(pochhammer(n + k, m - k, radix));
+    }
+
+    // Product of the numbers n * (n + 1) * (n + 2) * ... * (n + m - 1)
+    private static Apint pochhammer(Apint n, Apint m)
+    {
+        assert (m.signum() >= 0);
+        Apint one = Apint.ONES[n.radix()];
+        if (m.signum() == 0)
+        {
+            return one;
+        }
+        if (m.equals(one))
+        {
+            return n;
+        }
+        Apint two = new Apint(2, n.radix());
+        Apint k = m.divide(two);
+        return pochhammer(n, k).multiply(pochhammer(n.add(k), m.subtract(k)));
+    }
+
     /**
      * Product of numbers.
      * This method may perform significantly better
@@ -696,8 +889,8 @@ public class ApintMath
      *
      * @return A random number, uniformly distributed between <code>0 &le; x &lt; radix<sup>digits</sup></code>.
      *
-     * @exception java.lang.NumberFormatException If the default radix is not valid.
-     * @exception java.lang.IllegalArgumentException In case the number of specified digits is invalid.
+     * @exception NumberFormatException If the default radix is not valid.
+     * @exception IllegalArgumentException In case the number of specified digits is invalid.
      *
      * @since 1.9.0
      */
@@ -722,8 +915,8 @@ public class ApintMath
      *
      * @return A random number, uniformly distributed between <code>0 &le; x &lt; radix<sup>digits</sup></code>, in base <code>radix</code>.
      *
-     * @exception java.lang.NumberFormatException If the radix is not valid.
-     * @exception java.lang.IllegalArgumentException In case the number of specified digits is invalid.
+     * @exception NumberFormatException If the radix is not valid.
+     * @exception IllegalArgumentException In case the number of specified digits is invalid.
      *
      * @since 1.9.0
      */
