@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@ import org.apfloat.spi.Util;
  * <code>ApcomplexMath.acos(Apcomplex.ZERO)</code>.
  *
  * @since 1.5
- * @version 1.10.1
+ * @version 1.11.0
  * @author Mikko Tommila
  */
 
@@ -47,7 +47,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @param precision The precision of the results.
      *
-     * @exception java.lang.IllegalArgumentException In case the precision is invalid.
+     * @exception IllegalArgumentException In case the precision is invalid.
      */
 
     public FixedPrecisionApcomplexHelper(long precision)
@@ -152,7 +152,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z / w</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>w</code> is zero.
+     * @exception ArithmeticException If <code>w</code> is zero.
      */
 
     public Apcomplex divide(Apcomplex z, Apcomplex w)
@@ -169,7 +169,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z<sup>w</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> and <code>w</code> are zero.
+     * @exception ArithmeticException If <code>z</code> and <code>w</code> are zero.
      */
 
     public Apcomplex pow(Apcomplex z, Apcomplex w)
@@ -191,7 +191,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z<sup>n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> and <code>n</code> are zero, or <code>z</code> is zero and <code>n</code> is negative.
+     * @exception ArithmeticException If <code>z</code> and <code>n</code> are zero, or <code>z</code> is zero and <code>n</code> is negative.
      */
 
     public Apcomplex pow(Apcomplex z, long n)
@@ -207,7 +207,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return The angle of <code>z</code> on the complex plane.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is zero.
+     * @exception ArithmeticException If <code>z</code> is zero.
      */
 
     public Apfloat arg(Apcomplex z)
@@ -343,7 +343,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return The arc tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is <code>i</code>.
+     * @exception ArithmeticException If <code>z</code> is <code>i</code>.
      */
 
     public Apcomplex atan(Apcomplex z)
@@ -359,7 +359,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return The hyperbolic arc tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is 1 or -1.
+     * @exception ArithmeticException If <code>z</code> is 1 or -1.
      */
 
     public Apcomplex atanh(Apcomplex z)
@@ -431,7 +431,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return The natural logarithm of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is zero.
+     * @exception ArithmeticException If <code>z</code> is zero.
      */
 
     public Apcomplex log(Apcomplex z)
@@ -448,7 +448,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return The base-<code>w</code> logarithm of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> or <code>w</code> is zero.
+     * @exception ArithmeticException If <code>z</code> or <code>w</code> is zero.
      *
      * @since 1.6
      */
@@ -470,6 +470,11 @@ public class FixedPrecisionApcomplexHelper
     public Apcomplex sin(Apcomplex z)
         throws ApfloatRuntimeException
     {
+        Apcomplex result = checkSmallLinear(z);
+        if (result != null)
+        {
+            return result;
+        }
         return valueOf(ApcomplexMath.sin(setTrigExpPrecision(z)));
     }
 
@@ -484,6 +489,11 @@ public class FixedPrecisionApcomplexHelper
     public Apcomplex sinh(Apcomplex z)
         throws ApfloatRuntimeException
     {
+        Apcomplex result = checkSmallLinear(z);
+        if (result != null)
+        {
+            return result;
+        }
         return valueOf(ApcomplexMath.sinh(setExpTrigPrecision(z)));
     }
 
@@ -508,13 +518,18 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return The tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is &pi;/2 + n &pi; where n is an integer.
+     * @exception ArithmeticException If <code>z</code> is &pi;/2 + n &pi; where n is an integer.
      */
 
     public Apcomplex tan(Apcomplex z)
         throws ArithmeticException, ApfloatRuntimeException
     {
-        return valueOf(ApcomplexMath.tan(setTrigExpPrecision(z)));
+        Apcomplex result = checkSmallLinear(z);
+        if (result != null)
+        {
+            return result;
+        }
+        return valueOf(ApcomplexMath.tanFixedPrecision(setTrigExpPrecision(z)));
     }
 
     /**
@@ -524,12 +539,17 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return The hyperbolic tangent of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> is i (&pi;/2 + n &pi;) where n is an integer.
+     * @exception ArithmeticException If <code>z</code> is i (&pi;/2 + n &pi;) where n is an integer.
      */
 
     public Apcomplex tanh(Apcomplex z)
         throws ArithmeticException, ApfloatRuntimeException
     {
+        Apcomplex result = checkSmallLinear(z);
+        if (result != null)
+        {
+            return result;
+        }
         return valueOf(ApcomplexMath.tanhFixedPrecision(setExpTrigPrecision(z)));
     }
 
@@ -556,7 +576,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z<sup>-1/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> or <code>n</code> is zero.
+     * @exception ArithmeticException If <code>z</code> or <code>n</code> is zero.
      */
 
     public Apcomplex inverseRoot(Apcomplex z, long n)
@@ -574,7 +594,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z<sup>-1/n</sup>e<sup>-i2&pi;k/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>z</code> or <code>n</code> is zero.
+     * @exception ArithmeticException If <code>z</code> or <code>n</code> is zero.
      */
 
     public Apcomplex inverseRoot(Apcomplex z, long n, long k)
@@ -591,7 +611,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z<sup>1/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is zero, or <code>z</code> is zero and <code>n</code> is negative.
+     * @exception ArithmeticException If <code>n</code> is zero, or <code>z</code> is zero and <code>n</code> is negative.
      */
 
     public Apcomplex root(Apcomplex z, long n)
@@ -609,7 +629,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z<sup>1/n</sup>e<sup>i2&pi;sk/n</sup></code> where <code>s</code> is the signum of the imaginary part of <code>z</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is zero, or <code>z</code> is zero and <code>n</code> is negative.
+     * @exception ArithmeticException If <code>n</code> is zero, or <code>z</code> is zero and <code>n</code> is negative.
      */
 
     public Apcomplex root(Apcomplex z, long n, long k)
@@ -626,7 +646,7 @@ public class FixedPrecisionApcomplexHelper
      *
      * @return <code>z<sup>1/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is zero, or <code>z</code> is zero and <code>n</code> is negative.
+     * @exception ArithmeticException If <code>n</code> is zero, or <code>z</code> is zero and <code>n</code> is negative.
      */
 
     public Apcomplex[] allRoots(Apcomplex z, int n)
@@ -717,6 +737,166 @@ public class FixedPrecisionApcomplexHelper
         return valueOf(ApcomplexMath.sum(setPrecision(z)));
     }
 
+    /**
+     * <i>e</i>.
+     *
+     * @return <i>e</i>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat e()
+        throws ApfloatRuntimeException
+    {
+        return ApfloatMath.e(precision());
+    }
+
+    /**
+     * <i>e</i>.
+     *
+     * @param radix The radix of the result.
+     *
+     * @return <i>e</i>.
+     *
+     * @throws NumberFormatException If the radix is invalid.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat e(int radix)
+        throws NumberFormatException, ApfloatRuntimeException
+    {
+        return ApfloatMath.e(precision(), radix);
+    }
+
+    /**
+     * &gamma;, the Euler-Mascheroni constant.
+     *
+     * @return <code>&gamma;</code>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat euler()
+        throws ApfloatRuntimeException
+    {
+        return ApfloatMath.euler(precision());
+    }
+
+    /**
+     * &gamma;, the Euler-Mascheroni constant.
+     *
+     * @param radix The radix of the result.
+     *
+     * @return <code>&gamma;</code>.
+     *
+     * @throws NumberFormatException If the radix is invalid.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat euler(int radix)
+        throws NumberFormatException, ApfloatRuntimeException
+    {
+        return ApfloatMath.euler(precision(), radix);
+    }
+
+    /**
+     * Catalan's constant <i>G</i>.
+     *
+     * @return <i>G</i>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat catalan()
+        throws ApfloatRuntimeException
+    {
+        return ApfloatMath.catalan(precision());
+    }
+
+    /**
+     * Catalan's constant <i>G</i>.
+     *
+     * @param radix The radix of the result.
+     *
+     * @return <i>G</i>.
+     *
+     * @throws NumberFormatException If the radix is invalid.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat catalan(int radix)
+        throws NumberFormatException, ApfloatRuntimeException
+    {
+        return ApfloatMath.catalan(precision(), radix);
+    }
+
+    /**
+     * The Glaisher-Kinkelin constant <i>A</i>.
+     *
+     * @return <i>A</i>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat glaisher()
+        throws ApfloatRuntimeException
+    {
+        return ApfloatMath.glaisher(precision());
+    }
+
+    /**
+     * The Glaisher-Kinkelin constant <i>A</i>.
+     *
+     * @param radix The radix of the result.
+     *
+     * @return <i>A</i>.
+     *
+     * @throws NumberFormatException If the radix is invalid.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat glaisher(int radix)
+        throws NumberFormatException, ApfloatRuntimeException
+    {
+        return ApfloatMath.glaisher(precision(), radix);
+    }
+
+    /**
+     * Khinchin's constant <i>K</i>.
+     *
+     * @return <i>K</i>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat khinchin()
+        throws ApfloatRuntimeException
+    {
+        return ApfloatMath.khinchin(precision());
+    }
+
+    /**
+     * Khinchin's constant <i>K</i>.
+     *
+     * @param radix The radix of the result.
+     *
+     * @return <i>K</i>.
+     *
+     * @throws NumberFormatException If the radix is invalid.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat khinchin(int radix)
+        throws NumberFormatException, ApfloatRuntimeException
+    {
+        return ApfloatMath.khinchin(precision(), radix);
+    }
+
     /**
      * Gamma function.
      *
@@ -774,6 +954,236 @@ public class FixedPrecisionApcomplexHelper
         return valueOf(ApcomplexMath.gamma(setGammaPrecision(a), setPrecision(z0), setPrecision(z1)));
     }
 
+    /**
+     * Logarithm of the gamma function.
+     *
+     * @param z The argument.
+     *
+     * @return <code>log&Gamma;(z)</code>
+     *
+     * @throws ArithmeticException If <code>z</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex logGamma(Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.logGamma(setPrecision(z)));
+    }
+
+    /**
+     * Digamma function.
+     *
+     * @param z The argument.
+     *
+     * @return <code>&psi;(z)</code>
+     *
+     * @throws ArithmeticException If <code>z</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex digamma(Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.digamma(setPrecision(z)));
+    }
+
+    /**
+     * Binomial coefficient.
+     *
+     * @param n The argument.
+     * @param k The argument.
+     *
+     * @return The binomial coefficient.
+     *
+     * @throws ArithmeticException If <code>n</code> or <code>k</code> is negative.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat binomial(long n, long k)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApintMath.binomial(n, k));
+    }
+
+    /**
+     * Binomial coefficient.
+     *
+     * @param n The argument.
+     * @param k The argument.
+     * @param radix The radix.
+     *
+     * @return The binomial coefficient.
+     *
+     * @throws ArithmeticException If <code>n</code> or <code>k</code> is negative.
+     * @throws NumberFormatException If the radix is not valid.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat binomial(long n, long k, int radix)
+        throws ArithmeticException, NumberFormatException, ApfloatRuntimeException
+    {
+        return valueOf(ApintMath.binomial(n, k, radix));
+    }
+
+    /**
+     * Binomial coefficient.
+     *
+     * @param n The argument.
+     * @param k The argument.
+     *
+     * @return The binomial coefficient.
+     *
+     * @throws ArithmeticException If <code>n</code>, <code>k</code> or <code>n - k</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex binomial(Apcomplex n, Apcomplex k)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.binomial(setGammaPrecision(n), setGammaPrecision(k)));
+    }
+
+    /**
+     * Bernoulli number.
+     *
+     * @param n The argument.
+     *
+     * @return <code>B<sub>n</sub></code>
+     *
+     * @throws IllegalArgumentException If <code>n &lt; 0</code>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat bernoulli(long n)
+        throws IllegalArgumentException, ApfloatRuntimeException
+    {
+        return valueOf(AprationalMath.bernoulli(n));
+    }
+
+    /**
+     * Bernoulli number.
+     *
+     * @param n The argument.
+     * @param radix The radix.
+     *
+     * @return <code>B<sub>n</sub></code>
+     *
+     * @throws IllegalArgumentException If <code>n &lt; 0</code>.
+     * @throws NumberFormatException If the radix is not valid.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat bernoulli(long n, int radix)
+        throws IllegalArgumentException, NumberFormatException, ApfloatRuntimeException
+    {
+        return valueOf(AprationalMath.bernoulli(n, radix));
+    }
+
+    /**
+     * Riemann zeta function.
+     *
+     * @param s The argument.
+     *
+     * @return <code>&zeta;(s)</code>
+     *
+     * @throws ArithmeticException If <code>s</code> is <code>1</code>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex zeta(Apcomplex s)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.zeta(setZetaPrecision(s)));
+    }
+
+    /**
+     * Hurwitz zeta function.
+     *
+     * @param s The first argument.
+     * @param a The second argument.
+     *
+     * @return <code>&zeta;(s, a)</code>
+     *
+     * @throws ArithmeticException If <code>s</code> is <code>1</code> or if <code>a</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex zeta(Apcomplex s, Apcomplex a)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.zeta(setZetaPrecision(s), setPrecision(a)));
+    }
+
+    /**
+     * Confluent hypergeometric function <i><sub>0</sub>F<sub>1</sub></i>.
+     *
+     * @param a The first argument.
+     * @param z The second argument.
+     *
+     * @return <i><sub>0</sub>F<sub>1</sub>(; a; z)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex hypergeometric0F1(Apcomplex a, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.hypergeometric0F1(setPrecision(a), setPrecision(z)));
+    }
+
+    /**
+     * Kummer confluent hypergeometric function <i><sub>1</sub>F<sub>1</sub></i>.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param z The third argument.
+     *
+     * @return <i><sub>1</sub>F<sub>1</sub>(a; b; z)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex hypergeometric1F1(Apcomplex a, Apcomplex b, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.hypergeometric1F1(setPrecision(a), setPrecision(b), setPrecision(z)));
+    }
+
+    /**
+     * Hypergeometric function <i><sub>2</sub>F<sub>1</sub></i>.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param c The third argument.
+     * @param z The fourth argument.
+     *
+     * @return <i><sub>2</sub>F<sub>1</sub>(a, b; c; z)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public Apcomplex hypergeometric2F1(Apcomplex a, Apcomplex b, Apcomplex c, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.hypergeometric2F1(setPrecision(a), setPrecision(b), setPrecision(c), setPrecision(z)));
+    }
+
     /**
      * Unit in the last place.
      *
@@ -865,6 +1275,11 @@ public class FixedPrecisionApcomplexHelper
         return setTrigonometricPrecision(x);
     }
 
+    Apfloat setZetaPrecision(Apfloat x)
+    {
+        return setTrigonometricPrecision(x);
+    }
+
     private Apcomplex setPrecision(Apcomplex z)
         throws ApfloatRuntimeException
     {
@@ -904,5 +1319,19 @@ public class FixedPrecisionApcomplexHelper
         return z.precision(precision);
     }
 
+    private Apcomplex setZetaPrecision(Apcomplex z)
+    {
+        return new Apcomplex(setZetaPrecision(z.real()), setTrigonometricPrecision(z.imag()));
+    }
+
+    private Apcomplex checkSmallLinear(Apcomplex z)
+    {
+        if (z.scale() <= -precision())
+        {
+            return setPrecision(z);
+        }
+        return null;
+    }
+
     private long precision;
 }
