@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -37,7 +37,7 @@ import org.apfloat.spi.Util;
  * <code>ApfloatMath.acos(Apfloat.ZERO)</code>.
  *
  * @since 1.5
- * @version 1.10.0
+ * @version 1.11.0
  * @author Mikko Tommila
  */
 
@@ -50,7 +50,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @param precision The precision of the results.
      *
-     * @exception java.lang.IllegalArgumentException In case the precision is invalid.
+     * @exception IllegalArgumentException In case the precision is invalid.
      */
 
     public FixedPrecisionApfloatHelper(long precision)
@@ -140,7 +140,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return <code>x / y</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>y</code> is zero.
+     * @exception ArithmeticException If <code>y</code> is zero.
      */
 
     public Apfloat divide(Apfloat x, Apfloat y)
@@ -157,7 +157,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return <code>x<sup>y</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> and <code>y</code> are zero, or <code>x</code> is negative.
+     * @exception ArithmeticException If <code>x</code> and <code>y</code> are zero, or <code>x</code> is negative and <code>y</code> is not an integer.
      */
 
     public Apfloat pow(Apfloat x, Apfloat y)
@@ -168,6 +168,10 @@ public class FixedPrecisionApfloatHelper
         {
             return valueOf(result);
         }
+        else if (x.signum() < 0 && y.isInteger())
+        {
+            return valueOf(pow((Apcomplex) x, (Apcomplex) y).real());
+        }
         return exp(multiply(log(x), y));
     }
 
@@ -179,7 +183,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return <code>x<sup>n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> and <code>n</code> are zero, or <code>x</code> is zero and <code>n</code> is negative.
+     * @exception ArithmeticException If <code>x</code> and <code>n</code> are zero, or <code>x</code> is zero and <code>n</code> is negative.
      */
 
     public Apfloat pow(Apfloat x, long n)
@@ -209,7 +213,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The arc cosine of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If the absolute value of <code>x</code> is more than one.
+     * @exception ArithmeticException If the absolute value of <code>x</code> is more than one.
      */
 
     public Apfloat acos(Apfloat x)
@@ -230,7 +234,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The hyperbolic arc cosine of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If the <code>x</code> is less than one.
+     * @exception ArithmeticException If the <code>x</code> is less than one.
      */
 
     public Apfloat acosh(Apfloat x)
@@ -246,7 +250,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The arc sine of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If the absolute value of <code>x</code> is more than one.
+     * @exception ArithmeticException If the absolute value of <code>x</code> is more than one.
      */
 
     public Apfloat asin(Apfloat x)
@@ -290,7 +294,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The hyperbolic arc tangent of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If the absolute value of <code>x</code> is equal to or more than one.
+     * @exception ArithmeticException If the absolute value of <code>x</code> is equal to or more than one.
      */
 
     public Apfloat atanh(Apfloat x)
@@ -362,7 +366,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The natural logarithm of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> is less than or equal to zero.
+     * @exception ArithmeticException If <code>x</code> is less than or equal to zero.
      */
 
     public Apfloat log(Apfloat x)
@@ -379,7 +383,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The base-<code>b</code> logarithm of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> or <code>b</code> is less than or equal to zero.
+     * @exception ArithmeticException If <code>x</code> or <code>b</code> is less than or equal to zero.
      *
      * @since 1.6
      */
@@ -404,6 +408,11 @@ public class FixedPrecisionApfloatHelper
     public Apfloat sin(Apfloat x)
         throws ApfloatRuntimeException
     {
+        Apfloat result = checkSmallLinear(x);
+        if (result != null)
+        {
+            return result;
+        }
         return valueOf(ApfloatMath.sin(setTrigonometricPrecision(x)));
     }
 
@@ -418,6 +427,11 @@ public class FixedPrecisionApfloatHelper
     public Apfloat sinh(Apfloat x)
         throws ApfloatRuntimeException
     {
+        Apfloat result = checkSmallLinear(x);
+        if (result != null)
+        {
+            return result;
+        }
         return valueOf(ApfloatMath.sinh(setExponentialPrecision(x)));
     }
 
@@ -428,7 +442,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The square root of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> is negative.
+     * @exception ArithmeticException If <code>x</code> is negative.
      */
 
     public Apfloat sqrt(Apfloat x)
@@ -444,12 +458,17 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The tangent of <code>x</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> is &pi;/2 + n &pi; where n is an integer.
+     * @exception ArithmeticException If <code>x</code> is &pi;/2 + n &pi; where n is an integer.
      */
 
     public Apfloat tan(Apfloat x)
         throws ArithmeticException, ApfloatRuntimeException
     {
+        Apfloat result = checkSmallLinear(x);
+        if (result != null)
+        {
+            return result;
+        }
         return valueOf(ApfloatMath.tan(setTrigonometricPrecision(x)));
     }
 
@@ -464,6 +483,11 @@ public class FixedPrecisionApfloatHelper
     public Apfloat tanh(Apfloat x)
         throws ApfloatRuntimeException
     {
+        Apfloat result = checkSmallLinear(x);
+        if (result != null)
+        {
+            return result;
+        }
         return valueOf(ApfloatMath.tanhFixedPrecision(setExponentialPrecision(x)));
     }
 
@@ -490,7 +514,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return <code>x<sup>-1/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> or <code>n</code> is zero, or <code>x</code> is negative and <code>n</code> is even.
+     * @exception ArithmeticException If <code>x</code> or <code>n</code> is zero, or <code>x</code> is negative and <code>n</code> is even.
      */
 
     public Apfloat inverseRoot(Apfloat x, long n)
@@ -507,7 +531,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return <code>x<sup>1/n</sup></code>.
      *
-     * @exception java.lang.ArithmeticException If <code>n</code> is zero, or <code>x</code> is negative and <code>n</code> is even.
+     * @exception ArithmeticException If <code>n</code> is zero, or <code>x</code> is negative and <code>n</code> is even.
      */
 
     public Apfloat root(Apfloat x, long n)
@@ -607,6 +631,8 @@ public class FixedPrecisionApfloatHelper
     /**
      * Round with specified rounding mode.
      *
+     * @deprecated Use {@link #roundToPrecision(Apfloat,RoundingMode)}.
+     *
      * @param x The operand.
      * @param roundingMode The rounding mode.
      *
@@ -615,10 +641,89 @@ public class FixedPrecisionApfloatHelper
      * @since 1.7.0
      */
 
+    @Deprecated
     public Apfloat round(Apfloat x, RoundingMode roundingMode)
         throws ApfloatRuntimeException
     {
-        return ApfloatMath.round(x, precision(), roundingMode);
+        return roundToPrecision(x, roundingMode);
+    }
+
+    /**
+     * Round to precision with specified rounding mode.
+     *
+     * @param x The operand.
+     * @param roundingMode The rounding mode.
+     *
+     * @return <code>x</code> rounded to the precision of this helper with the specified rounding mode.
+     *
+     * @exception ArithmeticException If rounding is necessary (result is not exact) and rounding mode is {@link RoundingMode#UNNECESSARY}.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat roundToPrecision(Apfloat x, RoundingMode roundingMode)
+        throws ApfloatRuntimeException
+    {
+        return ApfloatMath.roundToPrecision(x, precision(), roundingMode);
+    }
+
+    /**
+     * Round to integer with specified rounding mode.
+     *
+     * @param x The operand.
+     * @param roundingMode The rounding mode.
+     *
+     * @return <code>x</code> rounded to integer with the specified rounding mode.
+     *
+     * @exception ArithmeticException If rounding is necessary (result is not exact) and rounding mode is {@link RoundingMode#UNNECESSARY}.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat roundToInteger(Apfloat x, RoundingMode roundingMode)
+        throws ApfloatRuntimeException, ArithmeticException
+    {
+        return valueOf(ApfloatMath.roundToInteger(x, roundingMode));
+    }
+
+    /**
+     * Round to specified number of places with specified rounding mode.
+     *
+     * @param x The operand.
+     * @param places The number of places.
+     * @param roundingMode The rounding mode.
+     *
+     * @return <code>x</code> rounded to the specified number of places with the specified rounding mode.
+     *
+     * @exception ArithmeticException If rounding is necessary (result is not exact) and rounding mode is {@link RoundingMode#UNNECESSARY}.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat roundToPlaces(Apfloat x, long places, RoundingMode roundingMode)
+        throws ApfloatRuntimeException, ArithmeticException
+    {
+        return valueOf(ApfloatMath.roundToPlaces(x, places, roundingMode));
+    }
+
+    /**
+     * Round to multiple with specified rounding mode.
+     *
+     * @param x The number to round.
+     * @param y The integer multiple to round to.
+     * @param roundingMode The rounding mode.
+     *
+     * @exception ArithmeticException If rounding is necessary (result is not exact) and rounding mode is {@link RoundingMode#UNNECESSARY}.
+     *
+     * @return <code>x</code> rounded to the nearest multiple of <code>y</code> with the specified rounding mode.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat roundToMultiple(Apfloat x, Apfloat y, RoundingMode roundingMode)
+        throws ApfloatRuntimeException, ArithmeticException
+    {
+        return valueOf(ApfloatMath.roundToMultiple(x, y, roundingMode));
     }
 
     /**
@@ -677,7 +782,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return The angle of the point <code>(y, x)</code>.
      *
-     * @exception java.lang.ArithmeticException If <code>x</code> and <code>y</code> are zero.
+     * @exception ArithmeticException If <code>x</code> and <code>y</code> are zero.
      */
 
     public Apfloat atan2(Apfloat x, Apfloat y)
@@ -793,7 +898,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return <code>&pi;</code>.
      *
-     * @exception java.lang.NumberFormatException If the radix is invalid.
+     * @exception NumberFormatException If the radix is invalid.
      */
 
     @Override
@@ -810,7 +915,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return <code>log(radix)</code>.
      *
-     * @exception java.lang.NumberFormatException If the radix is invalid.
+     * @exception NumberFormatException If the radix is invalid.
      */
 
     public Apfloat logRadix(int radix)
@@ -881,36 +986,6 @@ public class FixedPrecisionApfloatHelper
         return valueOf(ApfloatMath.sum(setPrecision(x)));
     }
 
-    /**
-     * &gamma;, the Euler-Mascheroni constant.
-     *
-     * @return <code>&gamma;</code>.
-     *
-     * @since 1.10.0
-     */
-
-    public Apfloat euler()
-        throws ApfloatRuntimeException
-    {
-        return ApfloatMath.euler(precision());
-    }
-
-    /**
-     * &gamma;, the Euler-Mascheroni constant.
-     *
-     * @param radix The radix of the result.
-     *
-     * @return <code>&gamma;</code>.
-     *
-     * @exception java.lang.NumberFormatException If the radix is invalid.
-     */
-
-    public Apfloat euler(int radix)
-        throws NumberFormatException, ApfloatRuntimeException
-    {
-        return ApfloatMath.euler(precision(), radix);
-    }
-
     /**
      * Gamma function.
      *
@@ -968,6 +1043,158 @@ public class FixedPrecisionApfloatHelper
         return valueOf(ApfloatMath.gamma(setGammaPrecision(a), setPrecision(x0), setPrecision(x1)));
     }
 
+    /**
+     * Logarithm of the gamma function.
+     *
+     * @param x The argument.
+     *
+     * @return <code>log&Gamma;(x)</code>
+     *
+     * @throws ArithmeticException If <code>x</code> is nonpositive.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat logGamma(Apfloat x)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.logGamma(setPrecision(x)));
+    }
+
+    /**
+     * Digamma function.
+     *
+     * @param x The argument.
+     *
+     * @return <code>&psi;(x)</code>
+     *
+     * @throws ArithmeticException If <code>x</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat digamma(Apfloat x)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.digamma(setPrecision(x)));
+    }
+
+    /**
+     * Binomial coefficient.
+     *
+     * @param n The argument.
+     * @param k The argument.
+     *
+     * @return The binomial coefficient.
+     *
+     * @throws ArithmeticException If <code>n</code>, <code>k</code> or <code>n - k</code> is a nonpositive integer.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat binomial(Apfloat n, Apfloat k)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.binomial(setGammaPrecision(n), setGammaPrecision(k)));
+    }
+
+    /**
+     * Riemann zeta function.
+     *
+     * @param s The argument.
+     *
+     * @return <code>&zeta;(s)</code>
+     *
+     * @throws ArithmeticException If <code>s</code> is <code>1</code>.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat zeta(Apfloat s)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.zeta(setZetaPrecision(s)));
+    }
+
+    /**
+     * Hurwitz zeta function.
+     *
+     * @param s The first argument.
+     * @param a The second argument.
+     *
+     * @return <code>&zeta;(s, a)</code>
+     *
+     * @throws ArithmeticException If <code>s</code> is <code>1</code> or if <code>a</code> is a nonpositive integer or if <code>s</code> is not an integer and <code>a</code> is nonpositive.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat zeta(Apfloat s, Apfloat a)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.zeta(setZetaPrecision(s), setPrecision(a)));
+    }
+
+    /**
+     * Confluent hypergeometric function <i><sub>0</sub>F<sub>1</sub></i>.
+     *
+     * @param a The first argument.
+     * @param x The second argument.
+     *
+     * @return <i><sub>0</sub>F<sub>1</sub>(; a; x)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat hypergeometric0F1(Apfloat a, Apfloat x)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.hypergeometric0F1(setPrecision(a), setPrecision(x)));
+    }
+
+    /**
+     * Kummer confluent hypergeometric function <i><sub>1</sub>F<sub>1</sub></i>.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param x The third argument.
+     *
+     * @return <i><sub>1</sub>F<sub>1</sub>(a; b; x)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat hypergeometric1F1(Apfloat a, Apfloat b, Apfloat x)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.hypergeometric1F1(setPrecision(a), setPrecision(b), setPrecision(x)));
+    }
+
+    /**
+     * Hypergeometric function <i><sub>2</sub>F<sub>1</sub></i>.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param c The third argument.
+     * @param x The fourth argument.
+     *
+     * @return <i><sub>2</sub>F<sub>1</sub>(a, b; c; x)</i>
+     *
+     * @throws ArithmeticException If the function value is not finite or real.
+     *
+     * @since 1.11.0
+     */
+
+    public Apfloat hypergeometric2F1(Apfloat a, Apfloat b, Apfloat c, Apfloat x)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.hypergeometric2F1(setPrecision(a), setPrecision(b), setPrecision(c), setPrecision(x)));
+    }
+
     /**
      * Return a uniformly distributed random number <code>0 &le; x &lt; 1</code>.
      *
@@ -989,7 +1216,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return A random number.
      *
-     * @exception java.lang.NumberFormatException If the radix is invalid.
+     * @exception NumberFormatException If the radix is invalid.
      *
      * @since 1.9.0
      */
@@ -1021,7 +1248,7 @@ public class FixedPrecisionApfloatHelper
      *
      * @return A random number.
      *
-     * @exception java.lang.NumberFormatException If the radix is invalid.
+     * @exception NumberFormatException If the radix is invalid.
      *
      * @since 1.9.0
      */
@@ -1147,4 +1374,13 @@ public class FixedPrecisionApfloatHelper
         }
         return tmp;
     }
+
+    private Apfloat checkSmallLinear(Apfloat x)
+    {
+        if (x.scale() <= -precision())
+        {
+            return setPrecision(x);
+        }
+        return null;
+    }
 }
